#pragma rtGlobals=1		// Use modern global access method.
#pragma version = 1.00

// This file uses constants defined by the procedure file DAQmxBaseConstants.ipf. You will need to locate it
// (in the folder More Extensions:Data Acquisition:NIDAQmxAccessXOP:) and open it before you can use these examples.

// All the examples are set to use this string constant as the name of the DAQ board. If you have only one board,
// this should do the trick. If you have more than one, you may need to edit this string.
static strconstant DeviceName = "dev1"

// At various places it is necessary to set an input mode. To make the code easier to modify the examples to suit your
// hardware requirements (some boards don't support RSE mode, some don't support NRSE, etc.), the mode is input
// as a constant rather than having it an input parameter to the function. By defining a constant, you only have to change
// it once, right here.

// You can't set a constant using a constant, so it is impossible to use the symbolic constants defined for this.
static constant AIMode = 10083

// 10083 corresponds to RSE (Referenced Single-Ended). Here are the definitions of the available input mode constants:
//constant DAQmx_Val_RSE =			10083 // RSE
//constant DAQmx_Val_NRSE =			10078 // NRSE
//constant DAQmx_Val_Diff =			10106 // Differential

// Analog outputs are particularly intolerant of min and max settings that don't match the range of the DAQ device.
// Most boards have analog outputs with a range of -10 to 10 volts. The USB 6008 has a range of just 0 to 5 V. Here are
// constants that make this easy to change. The first set should work for most devices:

//static constant AOMin = -10
//static constant AOMax = 10

// This set is appropriate for the USB 6008/9:
static constant AOMin = 0
static constant AOMax = 5

// *************************************
// a few functions to help keep the examples small and easy to read
// *************************************

// Uses error number to determine if the error is an Igor error or a DAQmxBase error.
// Calls the appropriate function to get a descriptive error message.
Function/S GetErrorMessageForMXBaseXOP(ErrorCode)
	Variable ErrorCode
	
	if (ErrorCode  > 10000 && ErrorCode  < 11000)
		return GetIgorMXBaseXOPErrorMsg(ErrorCode)
	else
		return GetDAQmxBaseErrorInfo()
	endif
end

Function/S GetDAQmxBaseErrorInfo()

	String info = ""
	DAQmxBaseXOP_GetErrorInfo(info)
	
	return info
end	

Function/S GetIgorMXBaseXOPErrorMsg(errorNumber)
	Variable errorNumber
	
	String info
	DAQmxBaseXOP_GetNonNIErrorInfo(errorNumber, info)
	return info
end

Function DoMXBaseErrorAlert(errorCode, abortCode)
	Variable errorCode
	Variable abortCode
	
	string einfo = GetErrorMessageForMXBaseXOP(ErrorCode)
		
	String errMsg = ""
	sprintf errMsg, "Error %d on line %d; More info?", errorCode, abortCode
	DoAlert 1, errMsg
	if (V_flag == 1)
		DoAlert 0, einfo
		print errMsg
		print einfo
	endif
end

// *************************************
// The examples
// *************************************

// ************ Analog input ***************

// Example taking a simple input voltage reading.
// In this example, a task is created, an analog input channel is added, the task is started and read, then destroyed.
// If you will be taking many readings of an analog input channel, it is better to create the task and start it. Then use
// DAQmxBaseXOP_ReadAnalogF64() for each reading. When you no longer need to read the analog inputs, stop and destroy the task.
Function ReadAnalogInput(chan)
	Variable chan
	
	Variable taskNumber
	Variable  errorCode
	Variable numSamplesRead
	Variable rate = 0.001
	
	Make/O/D/N=(1)  dummyReadWave = NaN
	
	// channels one at a time
	String ChanString = DeviceName+"/ai"+num2str(chan)
	
	try
		errorCode = DAQmxBaseXOP_CreateTask("SimpleScanTask", taskNumber); AbortOnValue errorCode != 0, 1
		errorCode = DAQmxBaseXOPCreateAIVoltageChan(taskNumber, ChanString, "AIReadingChannel", AIMode, -10, 10, DAQmx_Val_Volts, ""); AbortOnValue errorCode != 0, 2
		errorCode = DAQmxBaseXOP_StartTask(taskNumber); AbortOnValue errorCode != 0, 7	
		errorCode = DAQmxBaseXOP_ReadAnalogF64(taskNumber, 1, 10, DAQmx_Val_GroupByChannel, dummyReadWave, numSamplesRead); AbortOnValue errorCode != 0, 4
		errorCode = DAQmxBaseXOP_StopTask(taskNumber); AbortOnValue errorCode != 0, 5
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
	endtry

	DAQmxBaseXOP_ClearTask(taskNumber)
	Variable reading = dummyReadWave[0]
	KillWaves dummyReadWave
	return reading
end

// simple analog input scanning
// This function scans analog input channels ai0 and ai1. It reads 5000 samples from each channel at a rate of 1000 samples/s.
// The analog inputs are set to use Non-referenced single ended mode so that it will work correctly on the widest variety of boards. 
// If your board doesn't support Non-referenced single ended mode, you may need to change the definition of AIMode at the top of this procedure window.
Function ScanDouble2Chans()

	Variable taskNumber
	Variable  errorCode
	Variable numSamplesRead
	Variable rate = 0.001
	
	Variable totalSamples = 10000
	Variable SamplesPerChannel = 5000
	
	Make/O/D/N=(totalSamples)  BufferWave = NaN
	SetScale/P x 0,rate,BufferWave
	
	// channels one at a time
	String Chan0String = DeviceName+"/ai0"
	String Chan1String = DeviceName+"/ai1"
	
	// two channels together, using a channel range specification
	String TwoChanString = DeviceName+"/ai0:1"
	
	try
		// resetting the device first will shut down any other tasks that are running on your DAQ board. It can give you
		// a well-know state to start from, but it is not recommended that you do it at the level of creating an
		// individual task. It should be reserved for the initialization stage of an entire system.
//		errorCode = DAQmxBaseXOP_ResetDevice(DeviceName);AbortOnValue errorCode != 0, 0
		errorCode = DAQmxBaseXOP_CreateTask("SimpleScanTask", taskNumber); AbortOnValue errorCode != 0, 1
		// you can add the channels one-by-one
//		errorCode = DAQmxBaseXOPCreateAIVoltageChan(taskNumber, Chan0String, "Channel 0", AIMode, -10, 10, DAQmx_Val_Volts, ""); AbortOnValue errorCode != 0, 2
//		errorCode = DAQmxBaseXOPCreateAIVoltageChan(taskNumber, Chan1String, "Channel 1", AIMode, -10, 10, DAQmx_Val_Volts, ""); AbortOnValue errorCode != 0, 3
		// or both at the same time. This uses a range specification for the channels:
		errorCode = DAQmxBaseXOPCreateAIVoltageChan(taskNumber, TwoChanString, "Channel 0 and 1", AIMode, -10, 10, DAQmx_Val_Volts, ""); AbortOnValue errorCode != 0, 2
		errorCode = DAQmxBaseXOP_CfgSampClkTiming(taskNumber, "", 1/rate, DAQmx_Val_Rising, DAQmx_Val_FiniteSamps, SamplesPerChannel); AbortOnValue errorCode != 0, 6
		errorCode = DAQmxBaseXOP_StartTask(taskNumber); AbortOnValue errorCode != 0, 7	
		errorCode = DAQmxBaseXOP_ReadAnalogF64(taskNumber, SamplesPerChannel, 10, DAQmx_Val_GroupByChannel, BufferWave, numSamplesRead); AbortOnValue errorCode != 0, 4
print "samples read: ", numSamplesRead
		errorCode = DAQmxBaseXOP_StopTask(taskNumber); AbortOnValue errorCode != 0, 5
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
	endtry

	DAQmxBaseXOP_ClearTask(taskNumber)
end	

// trigger example
// This function scans analog input channels ai0 and ai1. It reads 1000 samples from each channel at a rate of 1000 samples/s.
// It starts scanning only after a trigger pulse is applied to PFI0 (pin 11 on most boards). The timeout is set to 100 seconds so that
// you have plenty of time to hit a button or do some other action that will make a trigger pulse.
// If your board doesn't support Non-referenced single ended mode, you may need to change the definition of AIMode at the top of this procedure window.
Function ScanDouble2ChansTrigger()

	Variable taskNumber
	Variable  errorCode
	Variable numSamplesRead
	Variable rate = 0.001
	Variable timeout = 100		// plenty of time to, for instance, push a button. If the trigger comes in, the scan will be read within a tenth of a second.
	
	Variable totalSamples = 2000
	Variable SamplesPerChannel = 1000
	
	Make/O/D/N=(totalSamples)  BufferWave = NaN
	SetScale/P x 0,rate,BufferWave
	
	String TwoChanString = DeviceName+"/ai0:1"
	String triggerSource = "/"+DeviceName+"/PFI0"
	
	try
		errorCode = DAQmxBaseXOP_CreateTask("SimpleScanTask", taskNumber); AbortOnValue errorCode != 0, 1
		errorCode = DAQmxBaseXOPCreateAIVoltageChan(taskNumber, TwoChanString, "Channel 0 and 1", AIMode, -10, 10, DAQmx_Val_Volts, ""); AbortOnValue errorCode != 0, 2
		errorCode = DAQmxBaseXOP_CfgSampClkTiming(taskNumber, "", 1/rate, DAQmx_Val_Rising, DAQmx_Val_FiniteSamps, SamplesPerChannel); AbortOnValue errorCode != 0, 3
		errorCode = DAQmxBaseXOP_CfgDigEdgeStrtTrig(tasknumber, triggerSource, DAQmx_Val_Rising); AbortOnValue errorCode != 0, 4
		errorCode = DAQmxBaseXOP_StartTask(taskNumber); AbortOnValue errorCode != 0, 5
		errorCode = DAQmxBaseXOP_ReadAnalogF64(taskNumber, SamplesPerChannel, 10, DAQmx_Val_GroupByChannel, BufferWave, numSamplesRead); AbortOnValue errorCode != 0, 6
print "samples read: ", numSamplesRead
		errorCode = DAQmxBaseXOP_StopTask(taskNumber); AbortOnValue errorCode != 0, 7
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
	endtry

	DAQmxBaseXOP_ClearTask(taskNumber)
end	

// Pre-trigger example.
// This function scans analog input channels ai0 and ai1. It reads 100 samples from each channel at a rate of 1000 samples/s.
// It starts scanning right away, but starts to count samples only after a trigger pulse is applied to PFI0 (pin 11 on most boards).
// It then scans 70 samples and stops, returning 30 pre-trigger samples.
// The timeout is set to 100 seconds so that you have plenty of time to hit a button or do some other action that will make a trigger pulse.
// If your board doesn't support Non-referenced single ended mode, you may need to change the definition of AIMode at the top of this procedure window.
// Not supported by USB 6008/9
Function ScanDouble2ChansPreTrigger()

	Variable taskNumber
	Variable  errorCode
	Variable numSamplesRead
	Variable rate = 0.001
	Variable timeout = 100		// plenty of time to, for instance, push a button. If the trigger comes in, the scan will be read within a tenth of a second.
	
	Variable totalSamples = 200
	Variable SamplesPerChannel = 100
	Variable preTriggerSamples = 30
	
	Make/O/D/N=(totalSamples)  BufferWave = NaN
	SetScale/P x 0,rate,BufferWave
	
	String TwoChanString = DeviceName+"/ai0:1"
	String triggerSource = "/"+DeviceName+"/PFI0"
	
	try
		errorCode = DAQmxBaseXOP_CreateTask("SimpleScanTask", taskNumber); AbortOnValue errorCode != 0, 1
		errorCode = DAQmxBaseXOPCreateAIVoltageChan(taskNumber, TwoChanString, "Channel 0 and 1", AIMode, -10, 10, DAQmx_Val_Volts, ""); AbortOnValue errorCode != 0, 2
		errorCode = DAQmxBaseXOP_CfgSampClkTiming(taskNumber, "", 1/rate, DAQmx_Val_Rising, DAQmx_Val_FiniteSamps, SamplesPerChannel); AbortOnValue errorCode != 0, 3
		errorCode = DAQmxBaseXOP_CfgDigEdgeRefTrig(tasknumber, triggerSource, DAQmx_Val_Rising, preTriggerSamples); AbortOnValue errorCode != 0, 4
		errorCode = DAQmxBaseXOP_StartTask(taskNumber); AbortOnValue errorCode != 0, 5
		errorCode = DAQmxBaseXOP_ReadAnalogF64(taskNumber, SamplesPerChannel, timeout, DAQmx_Val_GroupByChannel, BufferWave, numSamplesRead); AbortOnValue errorCode != 0, 6
print "samples read: ", numSamplesRead
		errorCode = DAQmxBaseXOP_StopTask(taskNumber); AbortOnValue errorCode != 0, 7
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
	endtry

	DAQmxBaseXOP_ClearTask(taskNumber)
end	

// simple analog input scanning, returns raw readings, rather than scaled voltage.
// This function scans analog input channels ai0 and ai1. It reads 5000 samples from each channel at a rate of 1000 samples/s.
// The analog inputs are set to use Non-referenced single ended mode so that it will work correctly on the widest variety of boards. 
// If your board doesn't support Non-referenced single ended mode, you may need to change the definition of AIMode at the top of this procedure window.
Function ScanInt2Chans()

	Variable taskNumber
	Variable  errorCode
	Variable numSamplesRead
	Variable rate = 0.001
	
	Variable totalSamples = 10000
	Variable SamplesPerChannel = 5000
	String TwoChanString = DeviceName+"/ai0:1"
	
	Make/O/W/N=(totalSamples)  IntBufferWave
	
	try
		errorCode = DAQmxBaseXOP_CreateTask("SimpleIntScanTask", taskNumber); AbortOnValue errorCode != 0, 1
		errorCode = DAQmxBaseXOPCreateAIVoltageChan(taskNumber, TwoChanString, "Channel 0", AIMode, -10, 10, DAQmx_Val_Volts, ""); AbortOnValue errorCode != 0, 2
		errorCode = DAQmxBaseXOP_CfgSampClkTiming(taskNumber, "", 1/rate, DAQmx_Val_Rising, DAQmx_Val_FiniteSamps, SamplesPerChannel); AbortOnValue errorCode != 0, 3
		errorCode = DAQmxBaseXOP_StartTask(taskNumber); AbortOnValue errorCode != 0, 4
		errorCode = DAQmxBaseXOP_ReadBinaryI16(taskNumber, SamplesPerChannel, 10, DAQmx_Val_GroupByChannel, IntBufferWave, numSamplesRead); AbortOnValue errorCode != 0, 5
print "samples read: ", numSamplesRead
		errorCode = DAQmxBaseXOP_StopTask(taskNumber); AbortOnValue errorCode != 0, 6
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
	endtry

	DAQmxBaseXOP_ClearTask(taskNumber)
end	



// ************ Analog output ***************

// Set an analog output to a specified voltage
// In this example, a task is created, an analog input channel is added, the task is started and read, then destroyed.
// If you will be setting the output voltage repeatedly, it is better to create the task and start it. Then use
// DAQmxBaseXOP_WriteAnalogF64() for each output voltage. When you no longer need to set the outputs, stop and destroy the task.
Function SetAnalogOutput(Chan, Volts)
	Variable Chan, Volts
	
	Variable taskNumber
	String ChanStr = DeviceName+"/ao"+num2str(Chan)
	Variable numsampswritten
	
	Make/D/O/N=1 dummyw=Volts
	
	Variable errorCode
	try
		errorCode = DAQmxBaseXOP_CreateTask("SimpleSetAOTask", taskNumber); AbortOnValue errorCode != 0, 1
		// In the next call, -10 and 10 are the min and max values. The documentation for these parameters states, "The minimum value, in volts, that you expect to generate."
		// I think it is supposed to select actual settings for your board that make sense for the board, that will give you the best resolution for the range you input.
		// In fact, at least with version 1.4.0f2 of NI-DAQmx Base, if I set anything other than -10 and 10 here, I get screwy voltages out. I'm using a PCI-6014;
		// if you are using something else, you may get different results.
		errorCode = DAQmxBaseXOPCreateAOVoltageChan(taskNumber, ChanStr, "", -10, 10, DAQmx_Val_Volts, ""); AbortOnValue errorCode != 0, 2
		// Here we start the task before writing to it. This conflicts with the advice in the NIDAQmxBaseAccess Help file. This is done because the
		// task is used for unbuffered, single-point output. This is because this type of output has no intermediate buffer. When generating waveforms,
		// there is an intermediate buffer which must be filled with a call to DAQmxBaseXOP_WriteAnalogF64 before starting the task.
		// Don't ask me- this is how the NI-DAQmx Base driver works!
		errorCode = DAQmxBaseXOP_StartTask(taskNumber); AbortOnValue errorCode != 0, 3.5
		// the third parameter below is autoStart. It must always be set to FALSE (0), which implies that you must always call DAQmxBaseXOP_StartTask()
		// before calling DAQmxBaseXOP_WriteAnalogF64. There is no DAQmxBaseXOP_WriteAnalogScalarF64(), so you have to make a 1-point dummy wave
		// and call the buffered (waveform generator) function DAQmxBaseXOP_WriteAnalogF64().
		errorCode = DAQmxBaseXOP_WriteAnalogF64(taskNumber, 1, 0, 1, DAQmx_Val_GroupByChannel, dummyw, numsampswritten); AbortOnValue errorCode != 0, 3
		errorCode = DAQmxBaseXOP_StopTask(taskNumber); AbortOnValue errorCode != 0, 4
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
	endtry

	DAQmxBaseXOP_ClearTask(taskNumber)
	
	KillWaves/Z dummyw
end

// This function scans two analog input channels while manipulating two analog output channels to make a "signal".
// It scans analog inputs 0 and 1 while manipulating analog output channels 0 and 1. The analog inputs are set to use
// referenced single ended mode as an attempt to make it work correctly on the widest variety of boards.
// If your board doesn't support referenced single ended mode, you may need to change the definition of AIMode at the top of this procedure window.
//
// To make the example work, you must make physical connections between the analog outputs and the analog inputs. On most boards,
// connect pin 22 (AO 0) to pin 68 (AI 0) and pin 21 (AO 1) to pin 33 (AI 1). 
//
// The Non-referenced single ended mode also requires connecting the analog input reference to analog ground. To do that, connect 
// pin 62 (AISENSE) to pin 29 (AIGND). If you change DAQmx_Val_NRSE to DAQmx_Val_RSE, you don't need to make that connection.

Function AnalogIOTest()

	Variable aiTask=-1		// initialize with a number that can't be a task number, so we can tell if it really contains a task number
	Variable ao0Task=-1
	Variable ao1Task=-1
	
	Make/D/O/N=2000 AnalogInputWave	// 1000 samples on two channels
	Make/D/O/N=1 dummyAOwave
	
	Variable numsampswritten
	Variable numSamplesRead
	
	// analog input channels
	String aichan0 = DeviceName+"/ai0"
	String aichan1 = DeviceName+"/ai1"
	
	// analog output channels
	String aochan0 = DeviceName+"/ao0"
	String aochan1 = DeviceName+"/ao1"
	
	Variable errorCode
	try
		// create a task to scan analog input channels 0 and 1
		errorCode = DAQmxBaseXOP_CreateTask("Test Task AI", aiTask); AbortOnValue errorCode != 0, 1
		errorCode = DAQmxBaseXOPCreateAIVoltageChan(aiTask, aichan0, "Channel 0", AIMode, -10, 10, DAQmx_Val_Volts, ""); AbortOnValue errorCode != 0, 2
		errorCode = DAQmxBaseXOPCreateAIVoltageChan(aiTask, aichan1, "Channel 1", AIMode, -10, 10, DAQmx_Val_Volts, ""); AbortOnValue errorCode != 0, 3
		// rate set to 100 so that we get 10 seconds of scanning with 1000 samples
		errorCode = DAQmxBaseXOP_CfgSampClkTiming(aiTask, "", 100, DAQmx_Val_Rising, DAQmx_Val_FiniteSamps, 1000); AbortOnValue errorCode != 0, 4
		
		// create two separate tasks to access the analog outputs
		errorCode = DAQmxBaseXOP_CreateTask("Test Task AO0", ao0Task); AbortOnValue errorCode != 0, 5
		errorCode = DAQmxBaseXOPCreateAOVoltageChan(ao0Task, aochan0, "", AOMin, AOMax, DAQmx_Val_Volts, ""); AbortOnValue errorCode != 0, 6
		dummyAOwave = 1		// set channel 0 to 1 volt
		errorCode = DAQmxBaseXOP_WriteAnalogF64(ao0Task, 1, 1, 1, DAQmx_Val_GroupByChannel, dummyAOwave, numsampswritten); AbortOnValue errorCode != 0, 7

		errorCode = DAQmxBaseXOP_CreateTask("Test Task AO1", ao1Task); AbortOnValue errorCode != 0, 8
		errorCode = DAQmxBaseXOPCreateAOVoltageChan(ao1Task, aochan1, "", AOMin, AOMax, DAQmx_Val_Volts, ""); AbortOnValue errorCode != 0, 9
		dummyAOwave = 4		// set channel 1 to 4 volts
		errorCode = DAQmxBaseXOP_WriteAnalogF64(ao1Task, 1, 1, 1, DAQmx_Val_GroupByChannel, dummyAOwave, numsampswritten); AbortOnValue errorCode != 0, 10

		// start scanning analog inputs
		errorCode = DAQmxBaseXOP_StartTask(aiTask); AbortOnValue errorCode != 0, 11
		
		// Now we make a series of changes of the analog output voltages. This gives us a "signal" to scan that doesn't depend on using waveform generation.
		// Waveform generation using DAQmx Base doesn't work on any board that lacks onboard FIFO memory.
		sleep/T 60
		dummyAOwave = 3
		errorCode = DAQmxBaseXOP_WriteAnalogF64(ao0Task, 1, 1, 1, DAQmx_Val_GroupByChannel, dummyAOwave, numsampswritten); AbortOnValue errorCode != 0, 12
		
		sleep/T 60
		dummyAOwave = 5
		errorCode = DAQmxBaseXOP_WriteAnalogF64(ao1Task, 1, 1, 1, DAQmx_Val_GroupByChannel, dummyAOwave, numsampswritten); AbortOnValue errorCode != 0, 13
		
		sleep/T 60
		// if your device (like a USB 6008/9) has a 5-volt analog output range, the next two voltages will be truncated:
		dummyAOwave = 6
		errorCode = DAQmxBaseXOP_WriteAnalogF64(ao0Task, 1, 1, 1, DAQmx_Val_GroupByChannel, dummyAOwave, numsampswritten); AbortOnValue errorCode != 0, 14
		
		sleep/T 60
		dummyAOwave = 8
		errorCode = DAQmxBaseXOP_WriteAnalogF64(ao1Task, 1, 1, 1, DAQmx_Val_GroupByChannel, dummyAOwave, numsampswritten); AbortOnValue errorCode != 0, 15
		
		sleep/T 60
		dummyAOwave = 0
		errorCode = DAQmxBaseXOP_WriteAnalogF64(ao0Task, 1, 1, 1, DAQmx_Val_GroupByChannel, dummyAOwave, numsampswritten); AbortOnValue errorCode != 0, 16
		
		sleep/T 60
		dummyAOwave = 0
		errorCode = DAQmxBaseXOP_WriteAnalogF64(ao1Task, 1, 1, 1, DAQmx_Val_GroupByChannel, dummyAOwave, numsampswritten); AbortOnValue errorCode != 0, 17
		
		// finished with our "signal", now do the analog input read
		errorCode = DAQmxBaseXOP_ReadAnalogF64(aiTask, 1000, 10, DAQmx_Val_GroupByChannel, AnalogInputWave, numSamplesRead); AbortOnValue errorCode != 0, 18
		print "Number of samples read per channel: ", numSamplesRead
		
		// finished with everything. Stop the tasks.
		errorCode = DAQmxBaseXOP_StopTask(aiTask); AbortOnValue errorCode != 0, 19
		errorCode = DAQmxBaseXOP_StopTask(ao0Task); AbortOnValue errorCode != 0, 20
		errorCode = DAQmxBaseXOP_StopTask(ao1Task); AbortOnValue errorCode != 0, 21		
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
	endtry
	
	// cleanup
	// It is possible for a task to not exist if an error occurs early in the function. That will result in one or more of these calls returning
	// a non-zero error code. But since we are shutting down, that's not really a problem...
	DAQmxBaseXOP_ClearTask(aiTask)
	DAQmxBaseXOP_ClearTask(ao0Task)
	DAQmxBaseXOP_ClearTask(ao1Task)
	
	KillWaves/Z dummyAOwave
end

// Generate a waveform on analog output channel 0.  The waveform is a 1000-point raised sinusoid having peak-to-peak amplitude of 4V.
// Not all DAQ devices can run this example. NI-DAQmx Base limits waveform generation to on-board waveform memory. Consequently,
// you can only generate waveforms on boards that *have* on-board memory for the purpose. That is actually rather rare.
//
// The only alternative is to write a loop that sets the output voltage using software timing. That is not easy to do, and it is not very accurate.
// 
Function WaveformTest(nPeriods)
	Variable nPeriods

	Variable nSamples = 1000
	Make/D/O/N=(nSamples) waveformWave
	SetScale/P x 0,0.001,waveformWave
	waveformWave = 2+2*sin(x*2*pi)
	Variable  sampleRate = 1/DimDelta(waveformWave, 0)

	Variable tasknumber
	Variable/G WaveFormTaskNumber
	String ChanStr = DeviceName+"/ao0"
	Variable numsampswritten
	Variable numSamplesToRun
	Variable sampleMode
	
	if (numtype(nPeriods) == 1)			// +- inf
		numSamplesToRun = nSamples		// probably not significant
		sampleMode = DAQmx_Val_ContSamps
	else
		numSamplesToRun = nSamples*nPeriods
		sampleMode = DAQmx_Val_FiniteSamps
	endif
	
	Variable errorCode
	try
		errorCode = DAQmxBaseXOP_CreateTask("SimpleSetAOTask", tasknumber); AbortOnValue errorCode != 0, 1
		WaveFormTaskNumber = tasknumber
		print "Analog output waveform task number is ", WaveFormTaskNumber
		errorCode = DAQmxBaseXOPCreateAOVoltageChan(WaveFormTaskNumber, ChanStr, "", AOMin, AOMax, DAQmx_Val_Volts, ""); AbortOnValue errorCode != 0, 2
		errorCode = DAQmxBaseXOP_CfgSampClkTiming(WaveFormTaskNumber, "", sampleRate, DAQmx_Val_Rising, sampleMode, numSamplesToRun); AbortOnValue errorCode != 0, 3
		errorCode = DAQmxBaseXOP_WriteAnalogF64(WaveFormTaskNumber, nSamples, 0, 1, DAQmx_Val_GroupByChannel, waveformWave, numsampswritten); AbortOnValue errorCode != 0, 4
		print "Analog output waveform samples written: ", numsampswritten
		errorCode = DAQmxBaseXOP_StartTask(WaveFormTaskNumber); AbortOnValue errorCode != 0, 5
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
		DAQmxBaseXOP_ClearTask(WaveFormTaskNumber)
	endtry
	
	KillWaves/Z dummyw
end

Function IsWaveformTaskDone()

	NVAR WaveFormTaskNumber

	Variable isDone
	Variable errorCode
	
	errorCode = DAQmxBaseXOP_IsTaskDone(WaveFormTaskNumber, isDone)
	if (errorCode)
		DoMXBaseErrorAlert(errorCode, 1)
		return NaN
	endif
	
	return isDone
end

Function ClearWaveformTaskIfDone()

	if (IsWaveformTaskDone())
		ClearWaveformTask()
	else
		print "Not done yet"
	endif
end

Function ClearWaveformTask()

	NVAR WaveFormTaskNumber
	
	Variable errorCode
	try
		errorCode = DAQmxBaseXOP_StopTask(WaveFormTaskNumber); AbortOnValue errorCode != 0, 1
		errorCode = DAQmxBaseXOP_ClearTask(WaveFormTaskNumber); AbortOnValue errorCode != 0, 2
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
		DAQmxBaseXOP_ClearTask(WaveFormTaskNumber)
	endtry
end

// *********** Load a task created by mxbaseconfig **********

// example of loading a task created using the mxbaseconfig  application.
// this function expects an analog input task with two channels. It makes a 20,000-point wave to receive data, so it
// will read 10,000 points per channel. It has a ten-second timeout, so make sure the task is fast enough to finish in 10 seconds.
// The name of the task is hard-coded to be "jw scan task"- you may need to change that.
Function LoadTaskTest()

	Variable taskNumber
	Variable nSamples = 20000
	Make/D/O/N=(nSamples) dummyw
	Variable SamplesPerChannel = nSamples/2
	
	Variable errorCode, numSamplesRead
	try
		errorCode = DAQmxBaseXOP_LoadTask("jw scan task", taskNumber); AbortOnValue errorCode != 0, 1
		errorCode = DAQmxBaseXOP_StartTask(taskNumber); AbortOnValue errorCode != 0, 2
		errorCode = DAQmxBaseXOP_ReadAnalogF64(taskNumber, SamplesPerChannel, 10, DAQmx_Val_GroupByChannel, dummyw, numSamplesRead); AbortOnValue errorCode != 0, 3
		print "Load task test: samples read: ", numSamplesRead
		errorCode = DAQmxBaseXOP_StopTask(taskNumber); AbortOnValue errorCode != 0, 4
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
	endtry
	DAQmxBaseXOP_ClearTask(tasknumber)
end

// *********** Digital I/O ************

// Read the state of a digital I/O  port
Function ReadDigitalPort(portnumber)
	Variable portnumber
	
	Variable taskNumber
	Variable  errorCode
	String portstr = DeviceName+"/port"+num2istr(portnumber)
		
	Variable linesetting = 0
	Variable i
	
	try
		errorCode = DAQmxBaseXOP_CreateTask("SimpleReadDigPortTask", taskNumber); AbortOnValue errorCode != 0, 1
		errorCode = DAQmxBaseXOP_CreateDIChan(taskNumber, portstr, "DI Channel", DAQmx_Val_ChanForAllLines); AbortOnValue errorCode != 0, 2
		errorCode = DAQmxBaseXOP_StartTask(taskNumber); AbortOnValue errorCode != 0, 3
		errorCode = DAQmxBaseXOP_ReadDigScalarU32(taskNumber, 1, linesetting); AbortOnValue errorCode != 0, 4
		errorCode = DAQmxBaseXOP_StopTask(taskNumber); AbortOnValue errorCode != 0, 5
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
	endtry
	
	DAQmxBaseXOP_ClearTask(taskNumber)
	
	return linesetting
end	

// Read the state of a single digital I/O  line.
// For many DAQ devices, DAQmx Base does not support digital I/O using individual lines; here we read the current state of the port,
// and return the state of the requested line using a bit mask. Use this for devices for which line-oriented reads and writes don't work.
// Works on port 0.
Function ReadDigitalLine(line)
	Variable line
	
	Variable taskNumber
	Variable  errorCode
	String portstr = DeviceName+"/port0"
		
	Variable linesetting = 0
	Variable lineMask = 2^line
	Variable i
	
	try
		errorCode = DAQmxBaseXOP_CreateTask("SimpleReadDigPortTask", taskNumber); AbortOnValue errorCode != 0, 1
		errorCode = DAQmxBaseXOP_CreateDIChan(taskNumber, portstr, "DI Channel", DAQmx_Val_ChanForAllLines); AbortOnValue errorCode != 0, 2
		errorCode = DAQmxBaseXOP_StartTask(taskNumber); AbortOnValue errorCode != 0, 3
		errorCode = DAQmxBaseXOP_ReadDigScalarU32(taskNumber, 1, linesetting); AbortOnValue errorCode != 0, 4
		errorCode = DAQmxBaseXOP_StopTask(taskNumber); AbortOnValue errorCode != 0, 5
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
	endtry
	
	DAQmxBaseXOP_ClearTask(taskNumber)
	
	return (linesetting & lineMask) != 0
end	

// Read the state of a single digital I/O  line.
// Uses line-oriented DIO calls- works on USB 6008/9 and M-series devices.
// Works on port 0.
Function ReadDigitalLine2(line)
	Variable line
	
	Variable taskNumber
	Variable  errorCode
	String portstr = DeviceName+"/port0/line"+num2istr(line)
		
	Variable linesetting = 0
	Variable i
	
	try
		errorCode = DAQmxBaseXOP_CreateTask("SimpleReadDigPortTask", taskNumber); AbortOnValue errorCode != 0, 1
		errorCode = DAQmxBaseXOP_CreateDIChan(taskNumber, portstr, "DI Channel", DAQmx_Val_ChanPerLine); AbortOnValue errorCode != 0, 2
		errorCode = DAQmxBaseXOP_StartTask(taskNumber); AbortOnValue errorCode != 0, 3
		errorCode = DAQmxBaseXOP_ReadDigScalarU32(taskNumber, 1, linesetting); AbortOnValue errorCode != 0, 4
		errorCode = DAQmxBaseXOP_StopTask(taskNumber); AbortOnValue errorCode != 0, 5
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
	endtry
	
	DAQmxBaseXOP_ClearTask(taskNumber)
	
	return linesetting
end	

Function ReadDigitalLineUsingWave(line)
	Variable line
	
	Variable taskNumber
	Variable  errorCode
	String portstr = DeviceName+"/port0"
		
	Make/O/B/U/N=1 linesettingw=0
	Variable lineMask = 2^line
	Variable i
	Variable sampsPerChanRead
	
	try
		errorCode = DAQmxBaseXOP_CreateTask("SimpleReadDigPortTask", taskNumber); AbortOnValue errorCode != 0, 1
		errorCode = DAQmxBaseXOP_CreateDIChan(taskNumber, portstr, "DI Channel", DAQmx_Val_ChanForAllLines); AbortOnValue errorCode != 0, 2
		errorCode = DAQmxBaseXOP_StartTask(taskNumber); AbortOnValue errorCode != 0, 3
		errorCode = DAQmxBaseXOP_ReadDigitalU8(taskNumber, 1, 1, DAQmx_Val_GroupByChannel, linesettingw, sampsPerChanRead); AbortOnValue errorCode != 0, 4

		errorCode = DAQmxBaseXOP_StopTask(taskNumber); AbortOnValue errorCode != 0, 5
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
	endtry
	
	DAQmxBaseXOP_ClearTask(taskNumber)
	
	Variable linesetting = linesettingw[0]
	KillWaves/Z linesettingw
	
	return (linesetting & lineMask) != 0
end	

// set the state of a digital I/O port
Function SetDigitalOut(linesetting)
	Variable linesetting
	
	Variable taskNumber
	Variable  errorCode
	String port0String = DeviceName+"/port0"
	
	try
		errorCode = DAQmxBaseXOP_CreateTask("SimpleSetDigOutTask", taskNumber); AbortOnValue errorCode != 0, 1
		errorCode = DAQmxBaseXOP_CreateDOChan(taskNumber, port0String, "DO Channel", DAQmx_Val_ChanForAllLines); AbortOnValue errorCode != 0, 2
		// second parameter is autoStart, set to TRUE. If you don't do that, you have to use DAQmxBaseXOP_StartTask() first
		errorCode = DAQmxBaseXOP_WriteDigScalarU32(taskNumber, 1, 1, linesetting); AbortOnValue errorCode != 0, 3
		errorCode = DAQmxBaseXOP_StopTask(taskNumber); AbortOnValue errorCode != 0, 4
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
	endtry
	
	DAQmxBaseXOP_ClearTask(taskNumber)
end

// set the state of a digital I/O port to turn a single line on or off
// For many DAQ devices, DAQmx Base does not support digital I/O using individual lines; here we read the current state of the port,
// calculate the new state with the specified line turned on or off, then write out the new port state.
Function SetDigitalOutLine(line, state)
	Variable line		// 0-7
	Variable state		// 0 for off, non-zero for on


	Variable taskNumber
	Variable  errorCode
	String port0String = DeviceName+"/port0"
	Variable lineMask = 2^line
	Variable linesetting
	
	try
		errorCode = DAQmxBaseXOP_CreateTask("SimpleSetDigOutTask", taskNumber); AbortOnValue errorCode != 0, 1
		errorCode = DAQmxBaseXOP_CreateDOChan(taskNumber, port0String, "DO Channel", DAQmx_Val_ChanForAllLines); AbortOnValue errorCode != 0, 2
		errorCode = DAQmxBaseXOP_StartTask(taskNumber); AbortOnValue errorCode != 0, 3
		
		// read current state of the port
		errorCode = DAQmxBaseXOP_ReadDigScalarU32(taskNumber, 1, linesetting); AbortOnValue errorCode != 0, 4
		// calculate new state with bit  set/cleared for the line we want to change
		if (state)
			lineSetting = linesetting | lineMask
		else
			lineSetting = linesetting & ~lineMask
		endif
		
		// write out new state of the port
		errorCode = DAQmxBaseXOP_WriteDigScalarU32(taskNumber, 0, 1, linesetting); AbortOnValue errorCode != 0, 5
		errorCode = DAQmxBaseXOP_StopTask(taskNumber); AbortOnValue errorCode != 0, 6
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
	endtry
	
	DAQmxBaseXOP_ClearTask(taskNumber)
end

// Set the state of a digital I/O port by manipulating an individual digital line.
// This function works only with devices for which line-oriented DIO is supported by DAQmx Base.
// That includes USB 6008/9, and M-series boards, but not E-series.
Function SetDigitalOutLine2(line, state)
	Variable line, state
	
	Variable taskNumber
	Variable  errorCode
	String port0String = DeviceName+"/port0/line"+num2istr(line)
	
	try
		errorCode = DAQmxBaseXOP_CreateTask("SimpleSetDigOutTask", taskNumber); AbortOnValue errorCode != 0, 1
		errorCode = DAQmxBaseXOP_CreateDOChan(taskNumber, port0String, "DO Channel", DAQmx_Val_ChanForAllLines); AbortOnValue errorCode != 0, 2
		// second parameter is autoStart, set to TRUE. If you don't do that, you have to use DAQmxBaseXOP_StartTask() first
		errorCode = DAQmxBaseXOP_WriteDigScalarU32(taskNumber, 1, 1, state!=0); AbortOnValue errorCode != 0, 3
		errorCode = DAQmxBaseXOP_StopTask(taskNumber); AbortOnValue errorCode != 0, 4
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
	endtry
	
	DAQmxBaseXOP_ClearTask(taskNumber)
end

// set the state of  a digital I/O port 0 inside a loop. 
// Note that the task  creation and destruction occurs outside the loop.
Function RotatingDigitalOut()

	Variable taskNumber
	Variable  errorCode
		
	Variable linesetting = 256
	Variable i
	String port0String = DeviceName+"/port0"
	
	try
		errorCode = DAQmxBaseXOP_CreateTask("SimpleRotatingDigOutTask", taskNumber); AbortOnValue errorCode != 0, 1
		errorCode = DAQmxBaseXOP_CreateDOChan(taskNumber, port0String, "DO Channel", DAQmx_Val_ChanForAllLines); AbortOnValue errorCode != 0, 2
		for (i  = 0; i < 100; i += 1)
			linesetting /= 2
			errorCode = DAQmxBaseXOP_WriteDigScalarU32(taskNumber, 1, 1, linesetting); AbortOnValue errorCode != 0, 3
			if (linesetting == 1)
				linesetting = 256
			endif
			sleep/T 6
		endfor
		errorCode = DAQmxBaseXOP_StopTask(taskNumber); AbortOnValue errorCode != 0, 4
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
	endtry
	
	DAQmxBaseXOP_ClearTask(taskNumber)
end

// ************ Counters ************

// The USB 6008/9 devices support only event counting. And the counter input is the same as the analog trigger input, so
// you can't do triggered analog input scans and count events at the same time.

// Function to start a pulse train output from a counter
// parameter numPulses could be used to generate a fixed number of pulses greater than 1 if we were using NI-DAQmx. But
// NI-DAQmx Base supports only continuous (numPulses=0) or a single pulse (numPulses=1). Asking for a finite number
// of pulses other than 1 results in odd behavior.
Function PulseTrain(counter, frequency, dutyCycle, numPulses)
	Variable counter
	Variable frequency, dutyCycle
	Variable numPulses		// zero means infinite
	
	Variable taskNumber
	Variable  errorCode
	String chanStr = DeviceName+"/ctr"+num2istr(counter)
		
	try
		errorCode = DAQmxBaseXOP_CreateTask("PulseOutputTask", taskNumber); AbortOnValue errorCode != 0, 1
		print "Pulse Train task number is ", taskNumber	// print out the task number so that you can later call StopPulseTrain()
		errorCode = DAQmxBaseXOPCreateCOPlsChanFreq(taskNumber, chanStr, "CounterPulseChan", DAQmx_Val_Hz, DAQmx_Val_Low, .7, frequency, dutyCycle); AbortOnValue errorCode != 0, 2
		if (numPulses == 0)
			errorCode = DAQmxBaseXOP_CfgImplicitTiming(taskNumber, DAQmx_Val_ContSamps, 1); AbortOnValue errorCode != 0, 3
		else
			errorCode = DAQmxBaseXOP_CfgImplicitTiming(taskNumber, DAQmx_Val_FiniteSamps, numPulses); AbortOnValue errorCode != 0, 4
		endif
		errorCode = DAQmxBaseXOP_StartTask(taskNumber); AbortOnValue errorCode != 0, 4
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
	endtry
	
	return taskNumber
end

// This function is intended for stopping a pulse train task started with PulseTrain(), above. In fact, it will stop *any* task,
// so use it carefully. A better way to do this would be to have PulseTrain() store the task number in  a global variable and to have
// StopPulseTrain() read the task number. That  way, it would stop only the correct task.
Function StopPulseTrain(taskNumber)
	Variable taskNumber
	
	Variable errorCode

	try
		errorCode = DAQmxBaseXOP_StopTask(taskNumber); AbortOnValue errorCode != 0, 1
		errorCode = DAQmxBaseXOP_ClearTask(taskNumber); AbortOnValue errorCode != 0, 2
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
	endtry
end

// ******** start of counter panel example ***********

// This example creates a small control panel that sets up Counter 0 to count edges and reads and displays the count 
// 10 times per second using a background task and SetVariable control. The panel includes a Start button, a Stop button
// and a Done button. The Done button stops and clears the counter task and destroys the control panel.

// This shows how to set up a counter for counting edges. It also shows the general technique of using a background task
// to do data acquisition. The background task can be used for periodically reading analog inputs, for example.

// The panel uses counter zero. If you don't have a convenient source of digital pulses to count, you can use counter 1
// to produce a pulse train (see PulseTrain(), above). In that case, on an E-series board or similar, connect 
// pin 40 (Ctr1Out) to pin 37 (Ctr0Src). USB 6008/9 devices have only one counter, so you will have to find some other source of pulses.

// To create the panel, execute SetupCounterPanel() on the command line. After that, it's all done by the buttons.

// create the control panel, set up the counter task and Igor background task
Function SetupCounterPanel()
	
	if (WinType("CounterPanel") == 7)	// the control panel already exists, just show it
		DoWindow/F CounterPanel
	else
		Variable/G counterContent = 0		// global variable used by the SetVariable display
		MakeCounterPanel()
	
		if (SetupCounter())
			DoAlert 0, "Something went wrong"
			return -1
		endif
		
		SetBackground CounterPanelBackgroundTask()
		CtrlBackground period = 6			// period of 6 ticks is a tenth of a second
	endif
	
	return 0
end

// Function to build the control panel
Function MakeCounterPanel()

	NewPanel /W=(45,171,201,319) as "Counter 0 Display"
	DoWindow/C CounterPanel
	
	SetVariable CounterContentDisplay,pos={21,63},size={108,15},title="Counter 0"
	SetVariable CounterContentDisplay,format="%d"
	// noedit=1 makes the SetVariable just a display. It cannot be used to change the value.
	SetVariable CounterContentDisplay,limits={-inf,inf,0},value= counterContent,noedit= 1,bodyWidth= 60
	Button CounterStopButton,pos={82,16},size={50,20},proc=CounterStopButtonProc,title="Stop"
	Button StartCounterPanelButton,pos={20,16},size={50,20},proc=CounterPanelStartButtonProc,title="Start"
	Button CounterDoneButton,pos={52,103},size={50,20},proc=CounterDoneButtonProc,title="Done"
EndMacro

// Sets up DAQmx Base counter task
Function SetupCounter()

	Variable taskNumber
	Variable  errorCode
	string counterChan = DeviceName+"/ctr0"
	
	try
		errorCode = DAQmxBaseXOP_CreateTask("CounterPanelTask", taskNumber); AbortOnValue errorCode != 0, 1
		// The USB 6008/9 support only DAQmx_Val_Falling; on other devices you can use DAQmx_Val_Rising as well.
		// Furthermore, it seems to support only counting up.
		errorCode = DAQmxBaseXOPCreateCICntEdgeChan(taskNumber, counterChan, "CountEdgesPanelChan", DAQmx_Val_Falling, 0, DAQmx_Val_CountUp); AbortOnValue errorCode != 0, 2
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
		
		return -1
	endtry

	// a global variable to communicate the tasknumber to the background task that refreshes the counter display
	Variable/G counterPanelTask = taskNumber
	return 0
end

// The background task that reads the counter and sets the value in the global variable displayed in the SetVariable display
Function CounterPanelBackgroundTask()

	NVAR counterPanelTask
	NVAR counterContent
	
	Variable errorCode
	Variable num=0
	
	try
		errorCode = DAQmxBaseXOP_ReadCntrScalarU32(counterPanelTask, .1, num);AbortOnValue errorCode != 0, 1
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
		
		DAQmxBaseXOP_StopTask(counterPanelTask)
		DAQmxBaseXOP_ClearTask(counterPanelTask)

		counterContent = NaN
		return 2
	endtry
	
	counterContent = num

	return 0
end

// The Start button action procedure. It calls DAQmxBaseXOP_StartTask() and starts the Igor background task.
Function CounterPanelStartButtonProc(ctrlName) : ButtonControl
	String ctrlName

	NVAR counterPanelTask
	Variable errorCode = DAQmxBaseXOP_StartTask(counterPanelTask)
	if (errorCode)
		DoMXBaseErrorAlert(errorCode, 1)
		DAQmxBaseXOP_StopTask(counterPanelTask)
		DAQmxBaseXOP_ClearTask(counterPanelTask)
		return -1
	endif
	
	CtrlBackground start
End

// The Stop button action procedure. It stops the counter task and the Igor background task.
Function CounterStopButtonProc(ctrlName) : ButtonControl
	String ctrlName

	NVAR counterPanelTask
	DAQmxBaseXOP_StopTask(counterPanelTask)
	CtrlBackground Stop
End

// The Done button action procedure. Call the Stop button procedure to shut down the counter
// and the background task, the clears (destroys) the counter task, and kills the control panel.
Function CounterDoneButtonProc(ctrlName) : ButtonControl
	String ctrlName

	NVAR counterPanelTask

	CounterStopButtonProc("")
	DAQmxBaseXOP_ClearTask(counterPanelTask)
	DoWindow/K CounterPanel
End

// ******** end of counter panel example ***********

// This example shows how to set up a buffered edge counting task. It reads a timed series of counter readings
// into a wave. Unfortunately, at least on my PCI-6014, it doesn't work well.
// Set source to the pin where the pulses appear. If you are using counter 0, that would probably be PFI9: "dev1/pfi9" for instance.
Function BufferedCountEdges(counternumber, source)
	Variable counternumber
	String source
	
	// The wave for a buffered edge-counting operation must be an unsigned integer wave
	Make/O/I/U/N=10  bufferWave=0
	SetScale/P x 0, 0.1, bufferWave
	
	Variable taskNumber
	Variable  errorCode
	string counterChan = DeviceName+"/ctr"+num2istr(counternumber)
	Variable npoints = numpnts(bufferWave)
	Variable rate = 1/DimDelta(bufferWave, 0 )
	Variable timeout = 1.5*npoints/rate			// set the timeout high enough to allow for problems...
	Variable sampsPerChanRead
	
	try
		errorCode = DAQmxBaseXOP_CreateTask("BufferedCountEdges", taskNumber); AbortOnValue errorCode != 0, 1
		print "Buffered count edges task number = ", tasknumber
		errorCode = DAQmxBaseXOPCreateCICntEdgeChan(taskNumber, counterChan, "BufferedCountEdgesChan", DAQmx_Val_Rising, 0, DAQmx_Val_CountUp); AbortOnValue errorCode != 0, 2
		errorCode = DAQmxBaseXOP_CfgSampClkTiming(taskNumber, source, rate, DAQmx_Val_Rising, DAQmx_Val_FiniteSamps, npoints); AbortOnValue errorCode != 0, 3
		errorCode = DAQmxBaseXOP_StartTask(taskNumber); AbortOnValue errorCode != 0, 4
		errorCode = DAQmxBaseXOP_ReadCounterU32(taskNumber, npoints, timeout, bufferWave, sampsPerChanRead);print "Samples read: ",sampsPerChanRead; AbortOnValue errorCode != 0, 5
		errorCode = DAQmxBaseXOP_StopTask(taskNumber); AbortOnValue errorCode != 0, 6
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
	endtry
	
	DAQmxBaseXOP_ClearTask(taskNumber)
end

// Example to use a counter to measure the period of a pulse train. DAQmxBaseXOP_CreateCIPeriodChan() does not
// have a source parameter, and DAQmx Base doesn't have a way to change the source. So you either have to connect it
// to the default pin (probably PFI9 or GPCTR0_GATE)  or use mxbaseconfig to create a task and then load it using
// DAQmxBaseXOP_LoadTask().
Function MeasurePeriod(counternumber)
	Variable counternumber

	Variable taskNumber
	Variable  errorCode
	string counterChan = DeviceName+"/ctr"+num2istr(counternumber)
	Variable sampsPerChanRead
	Variable value
	
	try
		errorCode = DAQmxBaseXOP_CreateTask("MeasurePeriod", taskNumber); AbortOnValue errorCode != 0, 1
		print "Period measurement task number = ", tasknumber
		errorCode = DAQmxBaseXOP_CreateCIPeriodChan(taskNumber, counterChan, "CounterPeriod", .000001, 0.1, DAQmx_Val_Seconds, DAQmx_Val_Rising, DAQmx_Val_LowFreq1Ctr, 0, 1, ""); AbortOnValue errorCode != 0, 2
		errorCode = DAQmxBaseXOP_StartTask(taskNumber); AbortOnValue errorCode != 0, 3
		// after starting the task, the first measurement may not include an entire period
		errorCode = DAQmxBaseXOP_ReadCntrScalarF64(taskNumber, 10, value); AbortOnValue errorCode != 0, 4
		print "Period: ", value
		errorCode = DAQmxBaseXOP_ReadCntrScalarF64(taskNumber, 10, value); AbortOnValue errorCode != 0, 4
		print "Period: ", value
		errorCode = DAQmxBaseXOP_StopTask(taskNumber); AbortOnValue errorCode != 0, 5
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
	endtry
	
	DAQmxBaseXOP_ClearTask(taskNumber)
end

// collects 10 period measurements into a wave. On my G5 using PCI-6014, it gives error -200141 "Data was overwritten before it could be read by the system."
// on any pulse rate over about 10 Hz, but it still gives correct results. Hmmm.....
Function MeasurePeriodBuffered(counternumber)
	Variable counternumber

	Variable taskNumber
	Variable  errorCode
	string counterChan = DeviceName+"/ctr"+num2istr(counternumber)
	Variable sampsPerChanRead
	Variable value
	
	// period measurement (and pulse width measurement) use a double-precision wave
	Make/O/N=10/D doubleWave = NaN
	Variable npnts = numpnts(doubleWave)
	Variable rate = 1/DimDelta(bufferWave, 0 )
	
	try
		errorCode = DAQmxBaseXOP_CreateTask("MeasurePeriod", taskNumber); AbortOnValue errorCode != 0, 1
		print "Period measurement task number = ", tasknumber
		errorCode = DAQmxBaseXOP_CreateCIPeriodChan(taskNumber, counterChan, "CounterPeriod", .000001, 0.1, DAQmx_Val_Seconds, DAQmx_Val_Rising, DAQmx_Val_LowFreq1Ctr, 0, 1, ""); AbortOnValue errorCode != 0, 2
		errorCode = DAQmxBaseXOP_CfgImplicitTiming(taskNumber, DAQmx_Val_FiniteSamps, npnts); AbortOnValue errorCode != 0, 3
		errorCode = DAQmxBaseXOP_StartTask(taskNumber); AbortOnValue errorCode != 0, 4
		errorCode = DAQmxBaseXOP_ReadCounterF64(taskNumber, npnts, 20, doubleWave, sampsPerChanRead); AbortOnValue errorCode != 0, 5
		print "Samples read: ", sampsPerChanRead, " Period: ", value
		errorCode = DAQmxBaseXOP_StopTask(taskNumber); AbortOnValue errorCode != 0, 6
	catch
		DoMXBaseErrorAlert(errorCode, V_AbortCode)
	endtry
	
	DAQmxBaseXOP_ClearTask(taskNumber)
end

